import numpy as np
import os
import math
from Bio.PDB import PDBIO, PDBList, PDBParser,Superimposer,Selection

# v1 - v4 are numpy vectors
def calc_dihedral(v1,v2,v3,v4):
   BA=v2-v1
   CB=v3-v2
   DC=v4-v3
   #print(BA,CB,DC)
   BAn = BA/np.linalg.norm(BA)
   CBn = CB/np.linalg.norm(CB)
   DCn = DC/np.linalg.norm(DC)
   ABC = np.cross(BA,CB)
   BCD = np.cross(CB,DC)
   n1=ABC/np.linalg.norm(ABC)
   n2=BCD/np.linalg.norm(BCD)
   # find the magnitude of the angle
   dihe = np.acos(np.dot(n1,n2))
   signAng = np.acos(np.dot(n1,n2))
   if signAng<0 :  
      dihe = -dihe
   return dihe
   
# Read the file 
def download_pdb(pdb_chain_id,pdir):   
    #pdb_code = pdb_chain_id.split("_")[0].lower()
    pdb_file = os.path.join(pdir, f"{pdb_chain_id}.pdb")

    if not os.path.exists("{pdb_chain_id}.pdb"):
        pdbl = PDBList()
        raw_path = pdbl.retrieve_pdb_file(pdb_chain_id, file_format='pdb', obsolete=False)
        # raw_path will be like: /content/pdb_pipeline/pdbs/pdbXXXX.ent
        new_path = os.path.join(pdir, f"{pdb_chain_id}.pdb")
        os.rename(raw_path, new_path)
    return pdb_file
    
path= download_pdb('2EZM','/home/nurith/CS612/homework/hw4/')
parser = PDBParser(PERMISSIVE = True, QUIET = True)
ezm = parser.get_structure("EZM", path)
fixed = Selection.unfold_entities(ezm, 'R')
v1 = fixed[8]["C"].get_coord()
v2 = fixed[9]["N"].get_coord()
v3 = fixed[9]["CA"].get_coord()
v4 = fixed[9]["C"].get_coord()
v5 = fixed[10]["N"].get_coord()
psi_dihe = calc_dihedral(np.array(v1),np.array(v2),np.array(v3),np.array(v4))
phi_dihe = calc_dihedral(np.array(v2),np.array(v3),np.array(v4),np.array(v5))
print(phi_dihe*180/math.pi)
print(psi_dihe*180/math.pi)

# This is the internal_coords solution for comparison only.
myChain = ezm[0]["A"]
ezm.atom_to_internal_coordinates(verbose=True)
for res in ezm.get_residues():
   if res.get_id()[1] == 10: 
      r1psi = res.internal_coord.get_angle("phi")
      r1phi = res.internal_coord.get_angle("psi")
      print(r1phi)
      print(r1psi)

