
mynorm<-function(v) {
	magn=sqrt(sum(v*v))
	result<-v/magn
	return (result)
}

# Cross product of two vectors
mycross<-function(v1,v2){
	xcomp<-v1[2]*v2[3]-v1[3]*v2[2]
	ycomp<-v1[3]*v2[1]-v2[3]*v1[1]
	zcomp<-v1[1]*v2[2]-v1[2]*v2[1]
	vec<-cbind(xcomp,ycomp,zcomp)
	return(vec)
}

# distance between two vectors
mydist<-function(v1,v2)
{
    return(dist(rbind(v1,v2)))
}

mysimpleene<-function(v1,v2)
{
  d<-dist(rbind(v1,v2))
  if(d > 3.4) 
  {
    return(0)    
  }
  else {
      #    print (d)
    ene<-100.0/(d*d)
    if(ene > 20)
      print( rbind("Clashing",d,ene))
    return(ene)
  }
}
# calculate the angle between two vectors, assuming normalized
myangle<-function(v1,v2){
return(acos(sum(v1*v2)))
}

dihedral<-function(v1,v2,v3,v4)
{
	BA=v2-v1
	CB=v3-v2
	DC=v4-v3
	BAn<-mynorm(BA)
	CBn<-mynorm(CB)
	DCn<-mynorm(DC)
	normABC<-mynorm(mycross(BAn,CBn))
	normBCD<-mynorm(mycross(CBn,DCn))
	dihe<-myangle(normABC,normBCD)
	signAng<-myangle(normABC,DCn)
 	if(signAng<0) {dihe = -dihe}
 	return(dihe)
}

quaternion1<-function(a,b,c,d)
{
	q<-c(a,b,c,d)
	return(q)
}

# Create quaternion from angle and axis
quaternion<-function(angle,axis)
{
	a<-cos(angle/2)
	axis=axis*sin(angle/2)
	q<-c(a,axis[,1],axis[,2],axis[,3])
	return(q)	
}

# Return the inverse of a quaternion
qinv<-function(q)
{
	qstar<-c(q[1],-(as.numeric(q[2])),-(as.numeric(q[3])),-(as.numeric(q[4])))
	return(qstar)
}

qproduct<-function(q1,q2)
{
    a1=q1[1]
    a2=q2[1]
    b1=q1[2]
    b2=q2[2]
    c1=q1[3]
    c2=q2[3]
    d1=q1[4]
    d2=q2[4]
    print(a1)
    a3<-a1*a2-b1*b2-c1*c2-d1*d2
    b3<-a1*b2+a2*b1+c1*d2-c2*d1
    c3<-a1*c2+a2*c1+b2*d1-b1*d2
    d3<-a1*d2+a2*d1+b1*c2-b2*c1
    q3<-c(a3,b3,c3,d3)
    return(q3)
}


qrotate<-function(p,h)
{
  hstar<-qinv(h)
  qp<-c(0,as.numeric(p[1]),as.numeric(p[2]),as.numeric(p[3])) # Convert p into a quaternion
  prod1<-qproduct(qp,hstar)
  prod2<-qproduct(h,prod1)
  return(c(prod2[2],prod2[3],prod2[4]))
}

# translate a matrix so that a certain index is the origin
mymatorig<-function(m,ind){
    v<-colMeans(m[ind,])
#    w<-rbind(v)
 #   newmat<-apply(m,1,function(x,y) x-y, y=v)
newmat<-sweep(m,MARGIN=2,v,FUN="-")
    return(newmat)
}

#main
myhw4q2a<-function(){
res10<-read.table("~/CS612/angle10.txt")
print(res10)
v1=as.vector(res10[1,])
v2=as.vector(res10[2,])
v3=as.vector(res10[3,])
v4=as.vector(res10[4,])
v5=as.vector(res10[5,])
print(v1,v2,v3,v4,v5)
phi<-dihedral(v1,v2,v3,v4)
psi<-dihedral(v2,v3,v4,v5)
vec<-c(phi*180/pi,psi*180/pi)
return(vec)
}

myhw4q2b<-function(){
coords<-read.table("~/CS612/coords.txt")
ene<-myene(coords)
return(ene)
}

# transform around an amino acid
myhw4q2c<-function(rotIndex,rotAngle){
    coords<-read.table("~/CS612/coords.txt")
    # Translate so axis is at origin
    mm<-mymatorig(coords,rotIndex)
    rotAxis=mm[rotIndex+1,]
    rotAxis=mynorm(rotAxis)
    q<-quaternion(rotAngle*3.1415/180.0,rotAxis)
  # Now rotate
    mm[rotIndex+2,]=qrotate(mm[rotIndex+2,],q)
    ene<-myene(mm)
    print(ene)
    write(as.matrix(mm),"~/CS612/coords_trans3c.txt", ncolumns=3)
    return(mm)
}
# transform around an amino acid
myhw4q2d<-function(rotIndex,rotAngle){
  coords<-read.table("~/CS612/coords.txt")
  # Translate so axis is at origin
  mm<-mymatorig(coords,rotIndex)
  write(as.matrix(mm),"~/CS612/coords_orig.txt", ncolumns=3)
  rotAxis=mm[rotIndex+1,]
  rotAxis=mynorm(rotAxis)
  q<-quaternion(rotAngle*3.1415/180.0,rotAxis)
  # Now rotate
  index=rotIndex+2
  index
  for(ind in index:303)
     mm[ind,]=qrotate(mm[ind,],q)
  ene<-myene(mm)
  print(ene)
  write(as.matrix(mm),"~/CS612/coords_trans3d.txt", ncolumns=3)
  return(mm)
}


# Calculate energy
myene<-function(coords)
{
size<-dim(coords)[1]
#c2<-coords[1:(size-4),]
ene<-0.0
#for(i in 1:(size-4)) {
for(j in 5:size) {
  c1<-coords[j:size,]
  ene<-ene+sum(apply(c1,1,function(x,y) mysimpleene(x,y), y=coords[j-4,]))
 }
return(ene)
#}
}

